"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startLiskCore = exports.validateStartCommandFlags = exports.getFinalConfigPath = exports.isLiskCoreV3Running = exports.installPM2 = exports.installLiskCore = exports.getLiskCoreStartCommand = void 0;
const tslib_1 = require("tslib");
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const fs = tslib_1.__importStar(require("fs-extra"));
const path = tslib_1.__importStar(require("path"));
const os_1 = require("os");
const fs_extra_1 = require("fs-extra");
const process_1 = require("./process");
const fs_1 = require("./fs");
const network_1 = require("./network");
const path_1 = require("./path");
const client_1 = require("../client");
const constants_1 = require("../constants");
const exception_1 = require("./exception");
const INSTALL_LISK_CORE_COMMAND = 'npm i -g lisk-core@^4.0.0-rc.1';
const INSTALL_PM2_COMMAND = 'npm i -g pm2';
const PM2_FILE_NAME = 'pm2.migrator.config.json';
const REGEX = {
    INPUT_SEPARATOR: /[\s=]+/,
    FLAG: /^(?:-[a-z]|--[a-z]+(?:-[a-z]+)*$)/,
    NETWORK_FLAG: /^(?:-n|--network)/,
    OPTION_OR_VALUE: /=<(option|value)>$/,
};
let liskCoreStartCommand;
const getLiskCoreStartCommand = () => liskCoreStartCommand;
exports.getLiskCoreStartCommand = getLiskCoreStartCommand;
const installLiskCore = async () => (0, process_1.execAsync)(INSTALL_LISK_CORE_COMMAND);
exports.installLiskCore = installLiskCore;
const installPM2 = async () => (0, process_1.execAsync)(INSTALL_PM2_COMMAND);
exports.installPM2 = installPM2;
const isLiskCoreV3Running = async (liskCorePath) => {
    try {
        const client = await (0, client_1.getAPIClient)(liskCorePath, true);
        const nodeInfo = await client.node.getNodeInfo();
        return !!nodeInfo;
    }
    catch (_) {
        return false;
    }
};
exports.isLiskCoreV3Running = isLiskCoreV3Running;
const backupLegacyDataDir = async (_this, liskCoreV3DataPath) => {
    try {
        if (!liskCoreV3DataPath.includes('.lisk/lisk-core')) {
            fs.mkdirSync(`${(0, os_1.homedir)()}/.lisk`, { recursive: true });
        }
        _this.log(`Backing Lisk Core v3 data directory at ${liskCoreV3DataPath}`);
        (0, fs_extra_1.renameSync)(liskCoreV3DataPath, constants_1.LISK_V3_BACKUP_DATA_DIR);
        _this.log(`Backed Lisk Core v3 data directory to: ${constants_1.LISK_V3_BACKUP_DATA_DIR}`);
    }
    catch (err) {
        throw new exception_1.MigratorException(`Unable to backup Lisk Core v3 data directory due to: ${err.message}`, 4);
    }
};
const copyLegacyDB = async (_this) => {
    try {
        _this.log(`Copying the v3.x snapshot to legacy.db at ${constants_1.LEGACY_DB_PATH}`);
        await (0, fs_1.copyDir)(path.resolve(constants_1.LISK_V3_BACKUP_DATA_DIR, constants_1.SNAPSHOT_DIR), (0, path_1.resolveAbsolutePath)(constants_1.LEGACY_DB_PATH));
        _this.log(`Legacy database for Lisk Core v4 has been created at ${constants_1.LEGACY_DB_PATH}`);
    }
    catch (err) {
        throw new exception_1.MigratorException(`Unable to copy ${path.basename(constants_1.LEGACY_DB_PATH)} due to: ${err.message}`, 5);
    }
};
const getFinalConfigPath = async (outputDir, network) => (await (0, fs_1.exists)(`${outputDir}/config.json`))
    ? outputDir
    : path.resolve(__dirname, '../..', 'config', network);
exports.getFinalConfigPath = getFinalConfigPath;
const validateStartCommandFlags = async (allowedFlags, userInputString) => {
    try {
        let isOptionOrValueExpected = false;
        const userInputs = userInputString.split(REGEX.INPUT_SEPARATOR);
        for (let i = 0; i < userInputs.length; i += 1) {
            const input = userInputs[i];
            if (REGEX.NETWORK_FLAG.test(input))
                return false;
            if (REGEX.FLAG.test(input)) {
                const correspondingFlag = allowedFlags.find(f => f.includes(input));
                if (!correspondingFlag || isOptionOrValueExpected || i === userInputs.length - 1) {
                    return false;
                }
                if (REGEX.OPTION_OR_VALUE.test(correspondingFlag)) {
                    isOptionOrValueExpected = true;
                }
            }
            else if (isOptionOrValueExpected) {
                isOptionOrValueExpected = false;
            }
            else {
                return false;
            }
        }
        return true;
    }
    catch (error) {
        return false;
    }
};
exports.validateStartCommandFlags = validateStartCommandFlags;
const resolveLiskCoreStartCommand = async (_this, network, configPath) => {
    const isUserConfirmed = await cli_ux_1.default.confirm('Would you like to customize the Lisk Core v4 start command? [yes/no]');
    const baseStartCommand = `lisk-core start --network ${network}`;
    if (!isUserConfirmed) {
        const defaultStartCommand = `${baseStartCommand} --config ${configPath}/config.json`;
        liskCoreStartCommand = defaultStartCommand;
        return defaultStartCommand;
    }
    let customStartCommand = baseStartCommand;
    _this.log('Customizing Lisk Core start command');
    let userInput = await cli_ux_1.default.prompt("Please provide the Lisk Core start command flags (e.g. --api-ws), except the '--network (-n)' flag:");
    const command = "lisk-core start --help | grep -- '^\\s\\+-' | cut -d ' ' -f 3,4";
    const allowedFlags = await (0, process_1.execAsync)(command);
    const allowedFlagsArray = allowedFlags.split(/\n+/).filter(e => !!e);
    let numTriesLeft = 3;
    while (numTriesLeft) {
        numTriesLeft -= 1;
        const isValid = await (0, exports.validateStartCommandFlags)(allowedFlagsArray, userInput);
        if (isValid) {
            customStartCommand = `${baseStartCommand} ${userInput}`;
            break;
        }
        if (numTriesLeft >= 0) {
            userInput = await cli_ux_1.default.prompt("Invalid flags passed. Please provide the Lisk Core start command flags (e.g. --api-ws), except the '--network (-n)' flag again:");
        }
        else {
            throw new Error('Invalid Lisk Core start command flags provided. Cannot proceed with Lisk Core v4 auto-start. Please continue manually. Exiting!!!');
        }
    }
    liskCoreStartCommand = customStartCommand;
    return customStartCommand;
};
const startLiskCore = async (_this, liskCoreV3DataPath, _config, network, outputDir) => {
    var _a, _b, _c, _d;
    try {
        const networkPort = (_b = (_a = _config === null || _config === void 0 ? void 0 : _config.network) === null || _a === void 0 ? void 0 : _a.port) !== null && _b !== void 0 ? _b : constants_1.DEFAULT_PORT_P2P;
        if (!(await (0, network_1.isPortAvailable)(networkPort))) {
            throw new Error(`Port ${networkPort} is not available for P2P communication.`);
        }
        const rpcPort = (_d = (_c = _config === null || _config === void 0 ? void 0 : _config.network) === null || _c === void 0 ? void 0 : _c.port) !== null && _d !== void 0 ? _d : constants_1.DEFAULT_PORT_RPC;
        if (!(await (0, network_1.isPortAvailable)(rpcPort))) {
            throw new Error(`Port ${rpcPort} is not available to start the RPC server.`);
        }
        await backupLegacyDataDir(_this, liskCoreV3DataPath);
        await copyLegacyDB(_this);
        const configPath = await (0, exports.getFinalConfigPath)(outputDir, network);
        const pm2Config = {
            name: 'lisk-core-v4',
            script: await resolveLiskCoreStartCommand(_this, network, configPath),
        };
        const isUserConfirmed = await cli_ux_1.default.confirm(`Start Lisk Core with the following pm2 configuration? [yes/no]\n${JSON.stringify(pm2Config, null, '\t')}`);
        if (!isUserConfirmed) {
            _this.error('User did not confirm to start Lisk Core v4 with the customized PM2 config. Skipping the Lisk Core v4 auto-start process. Please start the node manually.');
        }
        _this.log('Installing PM2...');
        await (0, exports.installPM2)();
        _this.log('Finished installing PM2.');
        const pm2FilePath = path.resolve(outputDir, PM2_FILE_NAME);
        _this.log(`Creating PM2 config at ${pm2FilePath}`);
        fs.writeFileSync(pm2FilePath, JSON.stringify(pm2Config, null, '\t'));
        _this.log(`Successfully created the PM2 config at ${pm2FilePath}`);
        const PM2_COMMAND_START = `pm2 start ${pm2FilePath}`;
        _this.log(await (0, process_1.execAsync)(PM2_COMMAND_START));
    }
    catch (err) {
        throw new exception_1.MigratorException(`${err.message}`, err instanceof exception_1.MigratorException ? err.code : 3);
    }
};
exports.startLiskCore = startLiskCore;
//# sourceMappingURL=node.js.map